@@ -100,7 +100,7 @@ ALLY_X_BUTTON_MAP: dict[int, GamepadButton] = to_map(
         "start": [EC("BTN_TR")],
         "select": [EC("BTN_TL")],
         # Misc
-        # "mode": [EC("BTN_MODE")],
+        "mode": [EC("BTN_SELECT")], # xbox
     }
 )
 
@@ -186,7 +186,7 @@ class AllyHidraw(GenericGamepadHidraw):
                             curr + MODE_DELAY,
                         )
                     )
-                case 0x38:
+                case 0x38 | 0x93:
                     # action = "right"
                     out.append({"type": "button", "code": "share", "value": True})
                     self.queue.append(
